#include <avr/io.h>

#include "usbdrv/oddebug.h"
#include "report.h"
#include "rb.h"
#include "ttable.h"
#include "mongodebug.h"

#define RB_SIZE 128
TYPE data[RB_SIZE];
rb_t scancodes;

enum P_STATE {
		ADD,
		REMOVE
} p_state = ADD;

static TYPE conv_fetch_uid(uint8_t scancode, bool extended)
{
	TYPE uid = 0;

	if (extended) {
		if (scancode > E_COUNT)
			return 0;
		uid = pgm_read_byte(&sc_hid_extended[scancode]);
	} else {
		if (scancode > S_COUNT)
			return 0;
		uid = pgm_read_byte(&sc_hid_standard[scancode]);
	}

	return uid;
}

void conv_parse()
{
	TYPE e = 0;
	TYPE uid = 0;
	static bool extended = false;

	if (rb_get(&scancodes, &e)) {
		if (e == 0xE0) {
			extended = true;
		} else if (e == 0xF0) {
			p_state = REMOVE;
		} else {
			uid = conv_fetch_uid(e, extended);
			extended = false;

			if (p_state == ADD) {
				report_add(uid);
			} else {
				p_state = ADD;
				report_remove(uid);
			}
		}
	}
}

void conv_init()
{
	rb_init(&scancodes, data, RB_SIZE);
}
