#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include <avr/pgmspace.h>
#include "usbdrv/usbdrv.h"
#include "usbdrv/oddebug.h"

//#include "soft_uart.h"
#include "misc.h"
#include "report.h"
#include "conv.h"
#include "ps2_uart.h"
#include "descriptor.h"
#include "mongodebug.h"

extern const report_t report;
static uchar idleRate; /* repeat rate for keyboards, never used for mice */

usbMsgLen_t usbFunctionSetup(uchar data[8])
{
	usbRequest_t *rq = (void *) data;

	/* The following requests are never used. But since they are required by
	 * the specification, we implement them in this example.
	 */
	if ((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS) { /* class request type */
		//DBG1(0x50, &rq->bRequest, 1); /* debug output: print our request */
		if (rq->bRequest == USBRQ_HID_GET_REPORT) { /* wValue: ReportType (highbyte), ReportID (lowbyte) */
			/* we only have one report type, so don't look at wValue */
			usbMsgPtr = (void *) &report;
			return sizeof(report);
		} else if (rq->bRequest == USBRQ_HID_GET_IDLE) {
			usbMsgPtr = &idleRate;
			return 1;
		} else if (rq->bRequest == USBRQ_HID_SET_IDLE) {
			idleRate = rq->wValue.bytes[1];
		}
	} else {
		/* no vendor specific requests implemented */
	}
	return 0; /* default for not implemented requests: return no data back to host */
}

static void init_vusb()
{
	uchar i;

	wdt_enable(WDTO_1S);
	usbInit();

	usbDeviceDisconnect(); /* enforce re-enumeration, do this while interrupts are disabled! */
	i = 0;
	while (--i) { /* fake USB disconnect for > 250 ms */
		wdt_reset();
		_delay_ms(1);
	}
	usbDeviceConnect();
}

int main(void)//__attribute__((noreturn)) main(void)
{
	report_init();
	conv_init();
	md_init();
	ps2_uart_init();
	init_vusb();
	sei();

	while (1) {
		wdt_reset();
		usbPoll();

		if (usbInterruptIsReady()) {
			if (report_changed())
				usbSetInterrupt((void*) &report, sizeof(report));
		}

		//ps2_uart_read();
		conv_parse();
	}
}
